;******************************************************************************

;* File Name	:alarm-gsm.asm

;* Title		:ALARM z powiadamianiem za pomoc SMS

;* Date	 		:14/05/2003

;* Version 		:1.0

;* Target MCU	:AT90S2313
;* 

;******************************************************************************
;
; zmienne w pamici EEPROM
; 
;-adres-------- opis -------- rozmiar ----- typ
; 0x00 		bajt statusu 		1 bajt		liczba 8-bit
; 0x01 		kod zaczenia 		4			tekst
; 0x05 		kod wyczenia		4			tekst
; 0x09 		numer telefonu		12			tekst+EOD
; 0x15 		numer pin			5			tekst+EOD
; 0x1A 		opisy czujnikw		8x9			tekst+EOD
; 0x62 		tre sms			29			tekst+EOD
;
.equ	STATUS_EEADDR 		= 0x00
.equ	ONCODE_EEADDR 		= 0x01
.equ	OFFCODE_EEADDR 		= 0x05
.equ	PINNUMBER_EEADDR 	= 0x09
.equ	SMSNUMBER_EEADDR 	= 0x0D
.equ	SENSOR1_EEADDR 		= 0x1B
.equ	SENSOR2_EEADDR 		= 0x23
.equ	SENSOR3_EEADDR 		= 0x2C
.equ	SENSOR4_EEADDR 		= 0x35
.equ	SENSOR5_EEADDR 		= 0x3E
.equ	SENSOR6_EEADDR 		= 0x47
.equ	SENSOR7_EEADDR 		= 0x50
.equ	SENSOR8_EEADDR 		= 0x59
.equ	SMSTEXT_EEADDR 		= 0x62
.equ	SENSORTEXT_LEN 		= 8
.equ	PHONENUM_LEN		= 11
.equ	SMSTEXT_LEN 		= 28
;------------------------------------------------------------------------------


.include "2313def.inc"

;stae zwizane z wywietlaczem
.equ	ENABLE 		= 7						;linia enable wywietlacza LCD
.equ	REGSEL 		= 4						;linia r/s wywietlacza LCD

;stae zwizane z UART i tablicami danych
.equ	CR 			= 0x0A					;kod znaku powrotu kursora
.equ	LF 			= 0x0D					;kod znaku nowej linii	
.equ	EOF 		= 0x1A					;kod znaku koca zbioru tekstowego
.equ	EOD 		= 0

;stae zwizane z klawiatur
.equ	COLUMN1 	= 4						;bit 1 kolumny klawiatury
.equ	COLUMN2 	= 5						;bit 2 kolumny klawiatury
.equ	COLUMN3 	= 6						;bit 3 kolumny klawiatury
.equ	NOKEY 		= 0xFF					;jeli "char" przyjmuje warto NOKEY, nie jest
											;wcinity aden klawisz
.equ	CLRKEY		= 10					;kod klawisza "CLR"
.equ	ENTERKEY	= 11					;kod klawisza "ENTER"
.equ	UCANSEE 	= 0xFF					;staa dla "visible" oznacza, e wprowadzane znaki mona czyta
.equ	UCANTSEE 	= 0x00					;staa dla "visible" oznacza, e wprowadzanych znakw nie mona czyta

;stae stanu alarmu
.equ	ALARMON 	= 0x0F					;status alarmu - zaczony
.equ	ALARMOFF 	= 0xF0					;status alarmu - wyczony
.equ	ALARMACTIVE = 0x55					;status alarmu - sygnalizacja
.equ	ALARMOUT 	= 6						;bit wyjcia portu B sterujcy zaczeniem alarmu
.equ	HIGHNIBBLE	= 6						;bit wyjcia portu D starujcy wyborem powki bajtu czujnikw
.equ	SENSORSOFF	= 0xFF					;aden z czujnikw nie jest wczony
.equ	DELAYTIME	= 30					;czas opnienia do zadziaania alarmu (w sekundach)
.equ	TIM1RLDH	= 0xE3
.equ	TIM1RLDL	= 0xDF					;stae do odwieania zawartoci Timer'a 1

;oglne definicje zmiennych pomocniczych
.dseg
.def	cchar1	= r02						;kod BCD znaku zwracanego przez funkcj "read_code" /1
.def	cchar2	= r03						; j.w. /2
.def	cchar3	= r04						; j.w. /3
.def	cchar4	= r05						; j.w. /4
.def	visible	= r06						;zmienna ma warto 1,gdy wprowadzany kod mona odczyta
.def	length	= r07						;dugo wprowadzanego tekstu dla read_text
.def	sensors	= r08						;stan sensorw (czujnikw) alarmu
.def	goon	= r09						;zmienna inkrementowana co 1 sek.-funkcja odmierzajca czas opnienia
.def	copypb	= r10						;przechowuje kopi portu b
.def	copypd	= r11						;przechowuje kopi portu d
.def	smssent	= r12						;warto rna od 0 oznacza,e SMS zosta wysany
.def	temp 	= r16						;zmienna pomocnicza
.def	delay1 	= r17						;dla procedury delay
.def	delay2 	= r18						; - / / -
.def	delay3	= r19						; - / / -
.def	delay 	= r20						;gwna zmienna dla delay - w niej podawany jest czas dla delayms
											;(w milisek.) i delaysec (w sekundach)
.def	i 		= r21						;zmienna pomocnicza - ptle i iteracje
.def	char 	= r22						;bufor znakw dla lcd
.def	eedata	= r23						;bajt do zapisu w eeprom
.def	eeaddr 	= r24						;adres zapisu bajtu
.def	sbuf	= r25						;bufor dla pojedynczego znaku odbieranego / wysyanego przez uart
;xp zawiera odcit, yp rzdn (definicja pokrywa si z "delay")
.def	xp 		= R17
.def	yp 		= R18

;zmienne (bufory) dla uart
buffer:	.byte 	80							;bufor dla znakw odebranych z uart lub zapisywanych/odczytywanych z eeprom

.cseg
.org	0x0000
;------------------------------------------------------------------------------
;wektory obsugi przerwa
;------------------------------------------------------------------------------

			rjmp	RESET					;Reset
			reti							;External Interrupt0
			reti							;External Interrupt1
			reti							;T/C1 Capture Event Interrupt
			reti							;T/C1 CompareA Interrupt
			rjmp	tim1_irq				;T/C1 Overflow Interrupt
			rjmp	tim0_irq				;T/C0 Overflow Interrupt
			reti							;UART Rx Complete Interrupt
			reti							;UART Data Register Empty Interrupt
			reti							;UART Tx Complete Interrupt
			reti							;Analog Comparator Interrupt



;-----------------------------------------------------------------------------
;mnoenie r0 x r1, wynik w r1:r0 (zamiast "mul",ktrego 2313 nie posiada)
;-----------------------------------------------------------------------------
mul_r0_r1:
			push	r15						;przechowanie r15,r16 i r17 na stosie-tu bd modyfikowane
			push	r16
			push	r17

			clr		r15						;zerowanie r15 i r16
			clr		r16
			mov		r17,r1					;jeli r1 jest 0,to mnoenie nie ma sensu (i wynik bdzie faszywy)
			tst		r17
			breq	mul_exit
mul_loop:
			add		r15,r0					;dodaj do r0 r15
			adc		r16,r2					;dodaj do r16 flag przeniesienia
			dec		r1						;powtrz r1 razy
			brne	mul_loop
mul_exit:
			mov		r0,r15					;przepisz wynik do R1:R0
			mov		r1,r16
			pop		r17
			pop		r16						;odtwrz r16 i r15
			pop		r15
			ret


;-----------------------------------------------------------------------------
;obsuga interfejsu szeregowego UART
;-----------------------------------------------------------------------------
uart_init:
			ldi		temp,47					;9600bps @ 7.3728MHz
			out		UBRR,temp	
			ret

;pobranie znaku z UART, znak zwracany w "sbuf"
getc:
			sbi		UCR,RXEN				;ustawienie flagi gotowoci do odbioru
			sbis	USR,RXC					;oczekiwanie na odbir znaku
			rjmp 	getc
			in		sbuf,UDR				;odczyt znaku do zmiennej temp
			cbi		UCR,RXEN				;zabronienie odbioru
			ret

;wysanie znaku, kod znaku przekazywany w sbuf
putc:
			sbi		UCR,TXEN				;zezwolenie na wysanie znaku
			sbis	USR,UDRE 				;oczekiwanie na oprnienie bufora
			rjmp 	putc
			out		UDR,sbuf				;wysanie znaku
			cbi		UCR,TXEN				;wyczenie interfejsu
			ret

;wysanie acucha znakw z pamici ROM przez UART, adres przekazywany w Z
;na kocu acucha spodziewany jest znak LF
putf:
			lpm								;R0 = (Z)
			mov		sbuf,r0
			cpi		sbuf,LF					;if (sbuf != LF) goto putf_next
			brne	putf_next
			rcall	putc					;wylij LF jako ostatni znak
			ret
putf_next:
			rcall	putc					;uart = sbuf
			adiw	zl,1					;Z = Z + 1 
			rjmp	putf					;goto putf

;wysanie acucha znakw z pamici RAM przez UART, adres przekazywany w Z
;na kocu acucha spodziewany jest znak LF
puts:
			ld		sbuf,z					;do "sbuf" znak spod adresu wskazywanego przez Z
			cpi		sbuf,LF					;jeli ten znak jest rny od LF,to pobierz nastpny
			brne	puts_next
			rcall	putc					;wylij LF jako ostatni znak
			ret
puts_next:
			rcall	putc					;wysanie "sbuf" przez UART
			inc		zl						;nastpny znak w pamici
			rjmp	puts

;odbir acucha znakw z UART
;znaki zapamitywane s w buforze w RAM
gets:
			ldi		zl,LOW(buffer)			;do Z adres bufora w pamici RAM
			clr		zh
gets_loop:
			rcall	getc					;pobierz znak z UART
			cpi		sbuf,LF					;jeli znak jest rny od LF,to dalej
			brne	gets_next
			st		z,sbuf					;zapamitaj LF jako ostatni odebrany znak
			ret
gets_next:
			st		z,sbuf					;zapamitaj "sbuf" w buforze w pamieci RAM
			inc		zl						;nastpna pozycja w RAM
			rjmp	gets_loop


;wysanie numeru PIN do telefonu:acuch trzeba "zoy" w pamici,poniewa numer PIN
;pobierany jest z EEPROM
send_PIN:
			ldi		zl,LOW(init_modem_2<<1)	;Z = adres napisu w ROM
			ldi		zh,HIGH(init_modem_2<<1)
			ldi		yl,LOW(buffer)			;Y = adres bufora w RAM
			clr		yh						;tu adresy RAM mog by tylko 1-bajtowe
			rcall	move_rom2ram
											;teraz pobieramy do bufora numer PIN z EEPROM
			ldi		eeaddr,PINNUMBER_EEADDR
			rcall	ee_read_code			;pobranie 4 bajtw do cchar1..4
			st		y,cchar1
			inc		r28						;zwikszenie indeksu Y
			st		y,cchar2
			inc		r28						;zwikszenie indeksu Y
			st		y,cchar3
			inc		r28						;zwikszenie indeksu Y
			st		y,cchar4

			rcall	st_EOL					;teraz koniec acucha - CR i LF do bufora
			rcall	puts_buffer				;wysanie do modemu "AT+CPIN=...."
			ret


;-----------------------------------------------------------------------------
;obsuga przerwania timera 0
;odpytywanie wej alarmu
;-----------------------------------------------------------------------------
tim0_irq:	push	temp					;zapamitanie "temp" na stosie
			cbi		portd,HIGHNIBBLE		;odczyt modszej powki bajtu wej czujnikw
			nop								;przerwa na czasy propagacji ukadw
			nop
			in		temp,pind				;odczyt modszej powki bajtu
			asr		temp					;"dopasowanie" bitw
			asr		temp
			mov		sensors,temp
			sbi		portd,HIGHNIBBLE
			nop								;przerwa na czasy propagacji ukadw
			nop
			in		temp,pind				;odczyt starszej powki bajtu
			asr		temp					;"dopasowanie" bitw
			asr		temp
			andi	temp,0x0F
			swap	temp
			or		sensors,temp
			pop		temp
			reti


;-----------------------------------------------------------------------------
;obsuga przerwania timera 1
;odmierzanie czasu opnienia do zadziaania alarmu
;-----------------------------------------------------------------------------
tim1_irq:
			sbi		portb,ALARMOUT			;zaczenie diody LED (migotanie w czasie "ALARMDELAY")
			push	temp					;przechowanie zawartoci "temp" na stosie
			ldi		temp,TIM1RLDH
			out		TCNT1H,temp
			ldi		temp,TIM1RLDL
			out		TCNT1L,temp				;odwieenie zawartoci Timer'a 1
			inc		goon					;zwikszenie licznika "goon"
			ldi		temp,0xFF
tim1_loop:
			rcall	delay1ms				;opnienie czasowe aby migotanie LED byo widoczne
			pop		temp
			cbi		portb,ALARMOUT			;wyczenie diody LED
			reti


;-----------------------------------------------------------------------------
;ptle opnienie czasowych
;-----------------------------------------------------------------------------
;procedura angauje cpu na czas okoo delay x 1ms
;czas w milisekundach w "delay"
delayms:
			push	xp						;zapamitanie "xp" i "yp",bo definicje maj wsplnie
			push	yp						;przydzielone rejestry r17..19
dloop0:
			ldi		delay1,0x0A
dloop2:
			ldi		delay2,0xF6
dloop1:
			dec		delay2					;pierwsza ptla 1/7.3728MHz x 127
			brne	dloop1
			dec		delay1					;druga ptla 1/7.3728MHz x 127 x 29
			brne	dloop2
			dec		delay					;testowanie parametru wywoania
			brne	dloop0

			pop		yp
			pop		xp
			ret

;opnienie 1 milisekunda
delay1ms:
			ldi		delay,1
			rjmp	delayms

;procedura angauje cpu na czas okoo delay x 1sekunda
;jako parametr wywoania podaje si czas w sekundach w "delay"
delaysec:
			mov		delay3,delay
			rol		delay3
			rol		delay3					;mnoenie x4
dsloop:
			ldi		delay,250
			rcall	delayms
			dec		delay3
			brne	dsloop
			ret

;-----------------------------------------------------------------------------
;obsuga wywietlacza LCD
;pb7 = enable, pb4 = r/s, pb0..3 = db4..7
;-----------------------------------------------------------------------------
;inicjalizacja wywietlacza LCD w trybie 4 bity
lcd_init:	cbi		portb,ENABLE
			cbi		portb,REGSEL			;ENABLE = REGSEL = 0;
			ldi		delay,15				;opnienie 15ms
			rcall	delayms
			ldi		i,3						;tutaj "i" jako licznik ptli
lcdiloop1:
			sbi		portb,ENABLE			;ENABLE=1;
			in		temp,portb				;portb = (pinb OR 0x0F) AND 0xF3;
			ori		temp,0x0F
			andi	temp,0xF3
			out		portb,temp
			cbi		portb,ENABLE			;ENABLE=0;
			ldi		delay,5					;opnienie 5ms
			rcall	delayms
			dec		i						;czy licznik osign warto 0?
			brne	lcdiloop1				;nie,kolejny obrt ptli

			sbi		portb,ENABLE			;ENABLE=1
			in		temp,portb				;temp = pinb
			ori		temp,0x0F				;portb = (temp | 0x0F) & 0xF2;
			andi	temp,0xF2				;ustawienie dolnej powki portu b
			out		portb,temp
			cbi		portb,ENABLE			;enable=0
			rcall	delay1ms				;opnienie 1ms

			ldi		char,0x28				;interfejs 4 bity,znaki 5x7
			rcall	lcd_cr_write
			ldi		char,0x08				;wyczenie LCD
			rcall	lcd_cr_write
			ldi		char,0x01				;kasowanie ekranu,powrt do home
			rcall	lcd_cr_write
			ldi		char,0x06				;przesuwanie kursora z inkrement.
			rcall	lcd_cr_write
			ldi		char,0x0C				;zaczenie wywietlacza
			rcall	lcd_cr_write
			ret
						
;zapis znaku do lcd, znak przekazywany w "char"
;procedura lcd_write_05 zapisuje 1/2 bajtu (starsz powk)
lcd_write_05:
			sbi		portb,ENABLE			;enable=1
			rcall	delay1ms				;opnienie 1 ms
			in		temp,portb				;portb |= 0x0F;
			ori		temp,0x0F				;portb &= ((char>>4) | 0xF0)
			mov		i,char					;operacji "char>>4" odpowiada instrukacja "swap char" w lcd_write
			ori		i,0xF0
			and		temp,i
			out		portb,temp
			cbi		portb,ENABLE			;enable=0
			rcall	delay1ms				;opnienie okoo 1ms
			ret

lcd_write:
			push	i
			swap	char
			rcall	lcd_write_05			;zapis starszej powki bajtu "char"
			swap	char
			rcall	lcd_write_05			;zapis modszej powki bajtu "char"
			pop		i
			ret

;zapis bajtu do rejestru kontrolnego wywietlacza
;bajt przekazywany w "char"
lcd_cr_write:
			cbi		portb,REGSEL
			rcall	lcd_write
			ret

;zapis bajtu pamiecie obrazu wywietlacza
;bajt przekazywany w "char"
lcd_putc:
			sbi		portb,REGSEL
			rcall	lcd_write
			ret

;ustawienie adresu pamici do zapisu bajtu odpowiednio do wsprzdnych x i y
lcd_goto_xy:
			push	char					;przechowanie "char" na stosie
			ldi		char,0xC0				;do "char" adres 2 linii
			sbrs	yp,0					;argumentem tej funkcji moe by "1" lub "0" tote sprawdzany jest tylko
											;bit numer 0 zmiennej "yp"
			ldi		char,0x80				;jeli bit 0 zm."yp" jest ustawiony,linia ta jest omijana-inaczej wpisywany
											;jest adres dla 1-szej linii LCD
			add		char,xp					;uwzgldnienie wsprzdnej "xp"
			rcall	lcd_cr_write			;ustawienie adresu w LCD
			pop		char
			ret

;kasowanie ekranu
lcd_clr_scr:
			ldi		char,0x01
			rcall	lcd_cr_write
			ret

;wywietlenie tekstu zapisanego w pamici programu
;rejestr Z (R31:R30) zawiera adres acucha w rom, ktrego koniec okrelany jest
;przez znak o kodzie 0x00
lcd_put_text:
			lpm								;R0 = flash(Z)
			tst		R0						;if (R0 = 0x00) goto lcd_put_next
			brne	lcd_put_next
			ret
lcd_put_next:
			mov		char,R0					;char = R0
			rcall	lcd_putc				;lcd = char
			adiw	zl,1					;Z = Z + 1 
			rjmp	lcd_put_text			;goto lcd_put_text

;zaczenie kursora z opcj "blink"
lcd_cursor_on:
			ldi		char,0x0F				;wczenie kursora
			rcall	lcd_cr_write
			ret

;wyczenie kursora i opcji blink
lcd_cursor_off:
			ldi		char,0x0C
			rcall	lcd_cr_write
			ret

;wywietlenie napisu "czekaj"
lcd_wait:
			rcall	lcd_clr_scr				;kasowanie ekranu
			ldi		zl,LOW(set_wait<<1)
			ldi		zh,HIGH(set_wait<<1)
			rcall	lcd_put_text
			ldi		delay,1					;przerwa po kadej z nastaw-czas na ustalenie klawiatury
			rcall	delaysec
			ret


;-----------------------------------------------------------------------------
;odczyt klawiatury
;-----------------------------------------------------------------------------
;odczyt wierszy - podprogram "kbd_read" (oczekiwana kolumna w stanie niskim)
kbd_row_read:
			nop
			nop								;czas na ustalenie si wyj portu b
			in		temp,pinb				;odczyt pinb, zmienna "temp" posuy do sprawdzenia czy jaki klawisz
			mov		char,temp				;zosta nacinity - "char" przechowuje kod klawisza
			andi	temp,0x0F				;maskowanie 4 starszych bitw
			cpi		temp,0x0F				;czy wcinity jest ktry klawisz?
			brne	kbd_row_read_again		;tak - powtrz odczyt
			ret								;aden klawisz nie jest wcinity - powrt
kbd_row_read_again:
			inc		i						;zwiksz licznik odczytw klawiatury
			cpi		i,20					;dla upewnienia si klawiatura czytana jest 20x
			brne	kbd_row_read
			ret

;odczytany znak zwracany w zmiennej char
kbd_read:
			push	i
			in		copypb,pinb				;kopia portu b
;sprawdzenie,czy ktrykolwiek klawisz jest wcinity-konieczne ze wzgldu na wsplne wyjcie 
;kolumny i sterowania przekanikiem 
			ldi		temp,0x0F				;ustawienie stanw wyjciowych portu b
			out		portb,temp
			rcall	kbd_row_read
			cpi		char,0x0F
			breq	kbd_read_NOKEY
			ldi		temp,0x7F
			out		portb,temp
;odczyt 1-szej kolumny klawiatury
			cbi		portb,COLUMN1			;ustawienie stanu niskiego na wyprowadzeniu kolumny 1
			clr		i						;zmienna "i" bdzie pracowa jako licznik odczytw
			rcall	kbd_row_read			;wywoanie podprogramu odczytu
			cpi		char,0x6F				;jeli nie wcinito klawisza, to zwracana jest warto maski (0x0F)
			brne	kbd_read_cases			;wwczas przejd do nastpnej kolumny

			sbi		portb,COLUMN1			;kolumna 1 = 1
			cbi		portb,COLUMN2			;kolumna 2 = 0
			clr		i
			rcall	kbd_row_read
			cpi		char,0x5F
			brne	kbd_read_cases

			sbi		portb,COLUMN2			;kolumna 2 = 1
			cbi		portb,COLUMN3			;kolumna 3 = 0
			clr		i
			rcall	kbd_row_read
			cpi		char,0x3F
			brne	kbd_read_cases
kbd_read_NOKEY:
			ldi		char,NOKEY				;jeli nie wcinito adnego klawisza, to zwracana jest warto 0xFF
			rjmp	kbd_read_exit
kbd_read_cases:
			cpi		char,0x67				;"1"
			brne	kbd_case_6E
			ldi		char,1
			rjmp	kbd_read_exit
kbd_case_6E:
			cpi		char,0x6E				;"4"
			brne	kbd_case_6D
			ldi		char,4
			rjmp	kbd_read_exit
kbd_case_6D:
			cpi		char,0x6D				;"7"
			brne	kbd_case_6B
			ldi		char,7
			rjmp	kbd_read_exit
kbd_case_6B:
			cpi		char,0x6B				;"0"
			brne	kbd_case_57
			clr		char
			rjmp	kbd_read_exit
kbd_case_57:
			cpi		char,0x57				;"2"
			brne	kbd_case_5E
			ldi		char,2
			rjmp	kbd_read_exit
kbd_case_5E:
			cpi		char,0x5E				;"5"
			brne	kbd_case_5D
			ldi		char,5
			rjmp	kbd_read_exit
kbd_case_5D:
			cpi		char,0x5D				;"8"
			brne	kbd_case_5B
			ldi		char,8
			rjmp	kbd_read_exit
kbd_case_5B:
			cpi		char,0x5B				;"C"
			brne	kbd_case_37
			ldi		char,CLRKEY
			rjmp	kbd_read_exit
kbd_case_37:
			cpi		char,0x37				;"3"
			brne	kbd_case_3E
			ldi		char,3
			rjmp	kbd_read_exit
kbd_case_3E:
			cpi		char,0x3E				;"6"
			brne	kbd_case_3D
			ldi		char,6
			rjmp	kbd_read_exit
kbd_case_3D:
			cpi		char,0x3D				;"9"
			brne	kbd_case_3B
			ldi		char,9
			rjmp	kbd_read_exit
kbd_case_3B:
			cpi		char,0x3B				;"ENTER"
			brne	kbd_read_exit
			ldi		char,ENTERKEY
kbd_read_exit:
			out		portb,copypb
			pop		i
			ret
			
;funkcja czyta z klawiatury kod 4-cyfrowy i zwraca go w komrka cchar1..4
;jeli kod ma by widoczny w czasie wprowadzania zmienna visible musi mie warto 0xFF
;spodziewane jest, e "xp" i "yp" zawiera bd wsprzdne pierwszego wywietlanego znaku
read_code:
			clt								;wycznie flagi "T"
			rcall	lcd_goto_xy
			clr		i						;for (i=0; i<5; i++) {
			ldi		zl,0x02					;Z = (*int)(R02)
			clr		zh
read_code_loop:
			brid	read_code_next			;jeli przerwania s wyczone,przejd do odczytu klawiatury
											;i nie sprawdzaj stanu sensorw
			mov		temp,sensors			;sprawdzenie stanu czujnikw!!! konieczne tutaj,bo funkcja 
											;nie przerywa pracy do momentu odczytu penego kodu
			cpi		temp,SENSORSOFF
			breq	read_code_next			;jeli aden sensor nie jest aktywny omi ponisze rozkazy
			brts	read_code_act			;jeli flaga T jest ustawiona,to nie wykonuj nastaw przerwania Timera 1
			cli								;wyczenie przerwa
			clr		goon					;kasowanie licznika czasu opnionego zaczenia (pracuje w przerw.Timer'a 1)
			ldi		temp,0x80
			out		TIMSK,temp				;zaczenie przerwa Timer'a 1 (co 1 sek.)
			sei								;zaczenie przerwa
read_code_act:
			set								;ustawienie flagi "T"
			push	zl						;przechowanie wskanika do bufora w RAM
			rcall	alarm_active
			pop		zl
read_code_next:
			rcall	kbd_read				;char=kbd_read()
			cpi		char,NOKEY				;if (char<>NOKEY) {
			brne	read_code_next1
			rjmp	read_code_loop
read_code_next1:
			cpi		char,CLRKEY				;klawisz CLRKEY jest ignorowany
			brsh	read_code_loop
			ldi		temp,'0'
			add		char,temp
			st		z,char					;*Z = char
			adiw	zl,1					;Z += i
			push	xp
			add		xp,i					;x = xp + i	
			rcall	lcd_goto_xy				;goto_xy(xp,yp)
			pop		xp
			mov		temp,visible
			cpi		temp,UCANSEE			;testowanie, czy uytkownik moe zaczy menu
			breq	read_code_char
			ldi		char,'*'				;jeli nie-wywietlany znak zastpowany jest "*"
read_code_char:
			rcall	lcd_putc				;wywietlenie znaku
			ldi		delay,250				;opnienie dla funkcji "autopowtarzania"
			rcall	delayms
			inc		i						;testowanie licznika wprowadzonych znakw
			cpi		i,4
			brne	read_code_loop
			ret

;funkcja odczytuje 11-cyfrowy numer dla SMS i zwraca go w "buffer"
;spodziewane jest, e "xp" i "yp" zawiera bd wsprzdne pierwszego wywietlanego znaku
read_number4sms:
			ldi		temp,PHONENUM_LEN		;dlugosc tekstu dla numeru telefonu
			mov		length,temp
			ldi		eeaddr,SMSNUMBER_EEADDR	;adres danych w eeprom
			rcall	read_text
			ret

;funkcja czyta tekst o dugoci "length" i zapamituje go w zmiennej "buffer"
;w funkcji uyto bufora UART "sbuf" jako zmiennej wskazujcej na konieczno zwikszenia indeksu "i"
;funkcja oczekuje,e ustawione bd zmienne xp, yp (pozycja 1-go znaku), length (dugo tekstu) oraz
;adres do zapisu danych w eeprom - eeaddr
read_text:
			rcall	lcd_goto_xy				;pozycjonowanie kursora na wsp.pocztkowych
			ldi		r26,0x55				;R26 wykorzystany do przechowywania ostatnio wprowadzonego znaku
			clr		r27						;R27 bdzie wskanikiem znaku w wierszu
			clr		i						;zmienna "i" pracuje jako licznik wprow.znakw
			clr		sbuf					;przy pierwszym wejciu kursor w pozycji 0,0 - nie zwikszamy "i"
			ldi		yl,LOW(buffer)			;rejestr Y zawiera wskanik do "buffer" w RAM
			clr		yh
read_text_loop:
			rcall	kbd_read				;czytaj znak z klawiatury
			cpi		char,NOKEY				;czy jaki klawisz by wcinity?
			brne	read_text_1				;tak-zapisz do bufora,nie-nastpna pozycja w buforze?
			tst		sbuf					;warto "1" oznacza, e naley zwikszy licznik "i"
			breq	read_text_loop
			clr		sbuf
			ldi		r26,0x55				;rozpoznawanie nastpnego klawisza
			inc		i						;nastpna pozycja w buforze
			cp		i,length				;porwnaj indeks "i" z parametrem okrelajcym maksymaln liczb znakw
			brlo	read_text_0				;jeli osignieto maksymaln liczb znakw nie pozwl na zwikszanie indeksu
			dec		i
			rjmp	read_text_loop
read_text_0:
			inc		yl						;wskazanie do nastpnej pozycji bufora w ram
			inc		xp						;nastpna pozycja na wywietlaczu
			cpi		xp,20					;czy przekroczono 19 znakw w wierszu?
			brlo	read_text_loop			;nie,wr do pocztku ptli
			clr		xp						;tak,wr na pocztek linii ekranu lcd
			inc		yp						;i zwiksz o 1 zmienn "yp" (nastpna linia)
			rcall	lcd_goto_xy				;pozycja kursora
			rjmp	read_text_loop			;wr do pocztku ptli
read_text_1:
											;rozpatrywanie otrzymanego znaku-jeli ten sam,nastpna pozycja wiersza
											;jeli inny kod,zmiana wiersza (znak zapamitany w 26
											;jeli wcinito klawisz "C" (kod LF),to kasuj ostatnio wprowadzony znak
											;jeli wcinieto kalwisz "ENTER" (CR),to zakocz wprowadzanie tekstu
;---------- klawisz CLR --------------
			cpi		char,CLRKEY
			brne	read_text_2
			ldi		delay,150				;opnienia dla kasowania znakw
			rcall	delayms
			tst		i						;sprawd,czy indeks pozwala na kasowanie znaku
			breq	read_text_loop			;osignito minimaln warto indeksu-powrt do pocztku ptli
			dec		i
			sbiw	r28,1					;zmniejsz wskazania indeksu bufora w RAM
			clr		temp
			st		y,temp					;zama znak w buforze w RAM
			dec		xp						;zmniejsz xp
			brmi	read_text_11			;jeli xp bdzie mniejsze od 0,to wyzeruj xp i zmniejsz yp
			rjmp	read_text_12
read_text_11:
			clr		xp
			dec		yp
read_text_12:
			rcall	lcd_goto_xy
			ldi		char,' '				;wywietl znak spacji
			rcall	lcd_putc
			rcall	lcd_goto_xy
			rjmp	read_text_loop

;---------- klawisz ENTER -------------
read_text_2:
			cpi		char,ENTERKEY
			brne	read_text_3
;			inc		yl
			clr		temp					;zapisanie znaku koca acucha
			st		y,temp
			rcall	ee_write_buf
			rcall	lcd_wait				;wywietlenie tekstu "czekaj..." i przerwa ok.1 sek.
			ret

;---------- inne klawisze -------------
read_text_3:
			push	char					;przechowanie warto "char" na stosie,bo pniej zamieniana na znak
			ldi		sbuf,1					;nowy znak-przy wyjciu naley zwikszy "xp"
			ldi		zl,LOW(chartable<<1)
			ldi		zh,HIGH(chartable<<1)	;zapamietanie w Z adresu tablicy w ROM
			cp		char,r26				;porwnanie poprzednio wprowadzonego znaku z biecym
			breq	read_text_4
			ldi		r27,0xFF				;zmieniono wiersz - indeks wiersza na 0xFF,co spowoduje przepenienie
read_text_4:								;podczas wykonywania operacji "inc r27" i w rezultacie wyzerowanie zmiennej
			inc		r27						;nastpna pozycja indeksu w wierszu
			cpi		r27,4
			brlo	read_text_41			;jeli indeks wiersza jest mniejszy od jego dugoci-dalej
			clr		r27						;w przeciwnym wypadku zeruj indeks
read_text_41:
			ldi		temp,4
			mov		r0,temp					;kod znaku (od 0..12) x "liczba znakw w wierszu" ->wynik w R1:R0
			mov		r1,char
			rcall	mul_r0_r1

			mov		temp,r0					;wyliczenie adresu napisu w pamici ROM
			add		temp,r27
			tst		temp					;jeli offset jest zerowy,to wyliczanie adresu tablic nie jest potrzebne
			breq	read_text_42
read_text_43:
			adiw	r30,1
			dec		temp
			brne	read_text_43
read_text_42:								;rejestr Z stanowi teraz wskanik do tablicy w ROM-stamtd pobieram znak
			lpm								;R0 = ROM(Z)
			mov		char,r0					;zapamitanie odczytanego znaku w zmiennej "char"
			rcall	lcd_goto_xy
			rcall	lcd_putc
			st		y,char					;zapisanie znaku do bufora
			pop		char					;przywrcenie wartoci wcinitego klawisza
			mov		R26,char				;zapamitanie wprowadzonego znaku
			ldi		delay,250				;opnienie dla "autorepeat"
			rcall	delayms
			rjmp	read_text_loop

;pytanie o poprawno wprowadzonych danych
;nacinicie ENTER ustawia flag Z, inne klawisze zeruj; funkcja czeka na zwolnienie klawisza
is_it_ok:
			clr		xp						;ustawienie kursora na wsp.0,0
			mov		yp,xp
			rcall	lcd_goto_xy
			ldi		zl,LOW(set_ok<<1)		;wywietlenie komunikatu-pytania
			ldi		zh,HIGH(set_ok<<1)
			rcall	lcd_put_text
is_it_ok_loop:
			rcall	kbd_read
			cpi		char,NOKEY
			breq	is_it_ok_loop
			cpi		char,ENTERKEY
			ret


;-----------------------------------------------------------------------------
;obsuga pamici eeprom
;-----------------------------------------------------------------------------
;zapis bajtu do EEPROM
;adres bajtu w eeaddr, bajt w eedata
ee_write:
			sbic	eecr,EEWE				;czekaj na EEWE = 0
			rjmp	ee_write
			out		eear,eeaddr				;podaj adres danych w eeprom
			out		eedr,eedata				;zapisz dane
			sbi		eecr,EEMWE				;ustaw bit "master ee write enable"
			nop
			sbi		eecr,EEWE				;ustaw bit "ee write enable"
			ret

;odczyt bajtu z EEPROM
;adres bajtu w eeaddr, bajt zwracany w eedata
ee_read:
			sbic	eecr,EEWE				;czekaj na EEWE = 0
			rjmp	ee_read
			out		eear,eeaddr				;podaj adres w eeprom
			sbi		eecr,EERE				;ustaw bit "ee read enable"
			in		eedata,eedr				;czytaj dane
			ret

;zapis danych z bufora w pamici RAM do EEPROM
;spodziewane: adres danych w eeaddr, koniec oznaczony jako EOD
ee_write_buf:
			ldi		yl,LOW(buffer)
			clr		yh
ee_write_loop:
			ld		eedata,y				;sbuf = (Y)
			cpi		eedata,EOD				;if (eedata != EOD) goto ee_write_next
			brne	ee_write_next
			rcall	ee_write				;zapisz EOD do eeprom
			ret
ee_write_next:
			rcall	ee_write				;zapisz bajt do eeprom
			inc		eeaddr					;eeadr = eeaddr+1
			inc		yl						;Y = Y + 1 
			rjmp	ee_write_loop			;goto ee_write_loop

;odczyt danych z EEPROM do bufora w pamici RAM
;adres danych w eeaddr, koniec oznaczony jako EOD, 
;adres miejsca,do ktrego przesyane s dane w rej.Y
ee_read2buf:
			rcall	ee_read
			cpi		eedata,EOD				;if (eedata != EOD) goto ee_read_next
			brne	ee_read_next
			st		y,eedata				;(Y) += EOD
			ret
ee_read_next:
			st		y,eedata
			inc		yl						;Y += 1
			inc		eeaddr					;eeadr += 1
			rjmp	ee_read2buf
			ret

;zapamietuje 4 komrki cchar1..4 w pamici eeprom
;na wejciu oczekiwany jest adres pocztkowy w eeprom podany w eeaddr
ee_save_code:
			mov		eedata,cchar1			;zapamitanie "cchar1"
			rcall	ee_write
			inc		eeaddr
			mov		eedata,cchar2			;zapamitanie "cchar2"
			rcall	ee_write
			inc		eeaddr
			mov		eedata,cchar3			;zapamitanie "cchar3"
			rcall	ee_write
			inc		eeaddr
			mov		eedata,cchar4			;zapamitanie "cchar4"
			rcall	ee_write
			ret

;odczytuje 4 komrki cchar1..4 z pamici eeprom
;na wejciu oczekiwany jest adres pocztkowy w eeprom podany w eeaddr
ee_read_code:
			rcall	ee_read
			mov		cchar1,eedata			;odczyt "cchar1"
			inc		eeaddr
			rcall	ee_read
			mov		cchar2,eedata			;odczyt "cchar2"
			inc		eeaddr
			rcall	ee_read
			mov		cchar3,eedata			;odczyt "cchar3"
			inc		eeaddr
			rcall	ee_read
			mov		cchar4,eedata			;odczyt "cchar4"
			ret

;wyznaczenie adresu etykiety wejcia alarmu w pamici eeprom
;w "eeaddr" jako argument podawany jest numer wejcia, ktry zamieniany jest na adres w eeprom 
;i zwracany rwnie w "eeaddr"
sen_ee_addr:
			push	i						;zmienna "i" jako licznik obrotw ptli
			mov		i,eeaddr				;przechowanie na stosie i inicjalizacja wartoci "i"
			ldi		eeaddr,SENSOR1_EEADDR	;nadanie wartoci "eeaddr"
			ldi		temp,SENSORTEXT_LEN		;do temp dugo opisu sensora
sen_ee_loop:
			dec		i						;zmniejszenie licznika ptli
			brne	sen_ee_next				;czy osign warto 0?
			pop		i						;tak,wyjcie z podprogramu i odtworzenie wartoci "i"
			ret
sen_ee_next:
			add		eeaddr,temp				;nie,dodaj do adresu bazowego dugo opisu sensora
			rjmp	sen_ee_loop


;-----------------------------------------------------------------------------
;alarm wyczony - oczekiwanie na kod zaczenia, tryb nastaw
;-----------------------------------------------------------------------------
;menu nastaw alarmu - do wywoania przy pierwszym uruchomieniu
alarm_set:
			rcall	lcd_cursor_on			;zaczenie kursora
			ldi		temp,UCANSEE			;zaczenie wywietlania wprowadzanych kodw
			mov		visible,temp
enter_code_on:
			rcall	lcd_clr_scr				;kod zaczenia
			ldi		zl,LOW(set_1<<1)
			ldi		zh,HIGH(set_1<<1)
			rcall	lcd_put_text
			clr		xp						;wsprzdne dla 1-go znaku
			ldi		yp,1
			rcall	read_code				;odczyt kodu zaczenia
			rcall	is_it_ok				;pytanie o poprawno
			brne	enter_code_on			;wywoywana jest funkcja pytajca o poprawno wprowadzonego numeru
											;jeli jest OK,to powrt nastpuje z ustawion flag Z
			ldi		eeaddr,ONCODE_EEADDR	;zapamitanie wprowadzonego kodu
			rcall	ee_save_code

enter_code_off:
			rcall	lcd_clr_scr				;kod wyczenia
			ldi		zl,LOW(set_2<<1)
			ldi		zh,HIGH(set_2<<1)
			rcall	lcd_put_text
			clr		xp						;wsprzdne dla 1-go znaku
			ldi		yp,1
			rcall	read_code				;odczyt kodu zaczenia
			rcall	is_it_ok				;pytanie o poprawno
			brne	enter_code_off			;wywoywana jest funkcja pytajca o poprawno wprowadzonego numeru
											;jeli jest OK,to powrt nastpuje z ustawion flag Z
			ldi		eeaddr,OFFCODE_EEADDR	;zapamitanie wprowadzonego kodu
			rcall	ee_save_code

enter_code_pin:
			rcall	lcd_clr_scr				;kod PIN
			ldi		zl,LOW(set_3<<1)
			ldi		zh,HIGH(set_3<<1)
			rcall	lcd_put_text
			clr		xp						;wsprzdne dla 1-go znaku
			ldi		yp,1
			rcall	read_code				;odczyt kodu pin
			rcall	is_it_ok				;pytanie o poprawno
			brne	enter_code_pin			;wywoywana jest funkcja pytajca o poprawno wprowadzonego numeru
											;jeli jest OK,to powrt nastpuje z ustawion flag Z
			ldi		eeaddr,PINNUMBER_EEADDR	;zapamitanie wprowadzonego kodu
			rcall	ee_save_code

			rcall	lcd_wait				;przerwa-wyeliminowanie skutku nacinicia ENTER
			rcall	lcd_clr_scr				;numer telefonu
			ldi		zl,LOW(set_4<<1)
			ldi		zh,HIGH(set_4<<1)
			rcall	lcd_put_text
			clr		xp						;wsprzdne dla 1-go znaku
			ldi		yp,1
			rcall	read_number4sms			;numer telefonu dla SMS

			rcall	lcd_clr_scr				;tekst SMS
			ldi		zl,LOW(set_5<<1)
			ldi		zh,HIGH(set_5<<1)
			rcall	lcd_put_text
			ldi		delay,2					;przerwa 3 sek., po niej kasowanie ekranu
			rcall	delaysec
			rcall	lcd_clr_scr

			ldi		temp,SMSTEXT_LEN		;ustawienie parametrw dla funkcji odczytu tekstu
			mov		length,temp
			ldi		eeaddr,SMSTEXT_EEADDR
			clr		xp
			mov		yp,xp
			rcall	read_text

			ldi		i,1						;zmienna "i" pracuje jako licznik ptli,jej warto musi by przechowywana
sen_eq_loop:								;etykieta czujnikw 1..8
			rcall	lcd_clr_scr				;kasowanie ekranu
			ldi		zl,LOW(set_6<<1)		;wpisanie adresu napisu do rejestru Z
			ldi		zh,HIGH(set_6<<1)
			rcall	lcd_put_text			;wywietlenie napisu z pamici ROM
			clr		yp
			ldi		xp,10
			rcall	lcd_goto_xy				;pozycjonowanie kursora
			ldi		char,'0'				;wywietlenie aktualnej wartoci licznika ptli "i"
			add		char,i
			rcall	lcd_putc
			ldi		temp,SENSORTEXT_LEN
			mov		length,temp				;wpisanie parametru dugoci napisu

			mov		eeaddr,i				;w "eeaddr" jako parametr numer komunikatu
			rcall	sen_ee_addr				;wyznaczenie adresu komunikatu w pamici EEPROM

			push	i						;przechowanie "i" (licznika) na stosie
			clr		xp						;wywietlenie kursora w miejscu wprowadzania tekstu
			ldi		yp,1
			rcall	read_text				;odczyt tekstu z klawiatury i zapamitanie w eeprom
			pop		i						;odtworzenie licznika
			inc		i						;kontrola,czy wykonano ju 8 "obrotw" ptli
			cpi		i,9
			brne	sen_eq_loop

			rcall	lcd_cursor_off			;wyczenie kursora
			ldi		eeaddr,STATUS_EEADDR	;ustawienie statusu alarmu na OFF
			ldi		eedata,ALARMOFF
			rcall	ee_write
			ret


;-----------------------------------------------------------------------------
;procedura pomocnicza - porwnanie kodu odczytanego przy pomocy "read_code"
;z kodem zapamitanym w eeprom; na wejciu spodziewany jest adres kodu w "eeaddr"
;i przy pmocy "read_code" kod w komrkach cchar1..4; jeli kody s rwne
;ustawiana jest flaga Z
;-----------------------------------------------------------------------------
compare_codes:
			push	cchar4					;zapamitanie odczytanego kodu na stosie
			push	cchar3
			push	cchar2
			push	cchar1
			rcall	ee_read_code			;do zmiennych cchar1..4

			pop		temp					;pobranie do "temp" cchar1
			sub		cchar1,temp				;odejmowanie od starego "cchar1"
			pop		temp					;pobranie do "temp" cchar1
			sub		cchar2,temp				;odejmowanie od starego "cchar2"
			pop		temp					;pobranie do "temp" cchar1
			sub		cchar3,temp				;odejmowanie od starego "cchar3"
			pop		temp					;pobranie do "temp" cchar1
			sub		cchar4,temp				;odejmowanie od starego "cchar4"

			mov		temp,cchar1				;sprawdzenie,czy wynikami poszczeglnych dziaa s "0"
			or		temp,cchar2
			or		temp,cchar3
			or		temp,cchar4
			ret

;-----------------------------------------------------------------------------
;tryb wprowadzania nastaw
;-----------------------------------------------------------------------------
alarm_parms:
			rcall	lcd_clr_scr				;kod wyczenia
			ldi		zl,LOW(set_2<<1)
			ldi		zh,HIGH(set_2<<1)
			rcall	lcd_put_text
			clr		xp						;wsprzdne dla 1-go znaku
			ldi		yp,1
			rcall	read_code				;odczyt kodu zaczenia

			ldi		eeaddr,OFFCODE_EEADDR
			rcall	compare_codes			;jeli kod odczytany jest identyczny z tym w eeprom,ustawiana jest fl.Z
			brne	alarm_params_exit
			rcall	alarm_set
alarm_params_exit:
			ret


;-----------------------------------------------------------------------------
;alarm wyczony - oczekiwanie na kod zaczenia
;-----------------------------------------------------------------------------
alarm_off:
			cli								;zabronienie przerwa
			rcall	lcd_clr_scr				;komunikat o wyczeniu
			ldi		zl,LOW(whenoff<<1)
			ldi		zh,HIGH(whenoff<<1)
			rcall	lcd_put_text
			clr		xp
			ldi		yp,1
			rcall	lcd_goto_xy
			ldi		zl,LOW(set_1<<1)
			ldi		zh,HIGH(set_1<<1)
			rcall	lcd_put_text
			ldi		temp,UCANTSEE
			mov		visible,temp
			ldi		temp,SENSORSOFF			;ustawienie statusu czujnikw na 0xFF,stan spoczynkowy
			mov		sensors,temp
off_loop:
			ldi		xp,16					;odczytanie kodu dla zaczenia z klawiatury
			ldi		yp,1
			rcall	read_code

			mov		temp,cchar1				;sprawdzenie,czy nie zosta wprowadzony kod "0000"
			or		temp,cchar2				;powoduje on wywoanie menu serwisowego
			or		temp,cchar3
			or		temp,cchar4
			subi	temp,0x30
			brne	of_loop_1
			rcall	alarm_parms
			ret
of_loop_1:
			ldi		eeaddr,ONCODE_EEADDR
			rcall	compare_codes			;jeli kod odczytany jest identyczny z tym w eeprom,ustawiana jest fl.Z
			brne	alarm_off

			ldi		eeaddr,STATUS_EEADDR	;pierwsze zaczenie alarmu
			ldi		eedata,ALARMON			;ustawienie statusu alarmu na OFF
			rcall	ee_write

			ret


;-----------------------------------------------------------------------------
;alarm zaczony - oczekiwanie na kod wyczenia, przegldanie
;stanu czujnikw w przerwaniu timer'a 0
;-----------------------------------------------------------------------------
alarm_on:
			ldi		i,DELAYTIME
			ldi		temp,0x80
			out		TIMSK,temp				;zaczenie przerwa Timer'a 1 (co 1 sekund)
			sei
			rcall	lcd_wait
al_on_wait:
			cp		temp,goon				;oczekiwanie a upynie czas przewidziany do zaczenia
			breq	al_on_wait
			sbi		portb,ALARMOUT			;krtki bysk diod
			rcall	delay1ms
			cbi		portb,ALARMOUT
			mov		temp,goon
			dec		i						;czy ju upyn czas DELAYTIME?
			brne	al_on_wait
			cli								;wyczenie przerwa
alarm_on_ww:
			clr		goon					;zerowanie licznika opnionego zaczenia
			clr		smssent					;nie wysano SMS z powiadomieniem
			rcall	lcd_clr_scr				;komunikat o zaczeniu
			ldi		zl,LOW(whenon<<1)
			ldi		zh,HIGH(whenon<<1)
			rcall	lcd_put_text
			clr		xp
			ldi		yp,1
			rcall	lcd_goto_xy
			ldi		zl,LOW(set_2<<1)
			ldi		zh,HIGH(set_2<<1)
			rcall	lcd_put_text
			ldi		temp,UCANTSEE			;wyczenie "widzialnoci" znakw
			mov		visible,temp

			ldi		temp,SENSORSOFF			;ustawienie statusu czujnikw na 0xFF,stan spoczynkowy
			mov		sensors,temp
			ldi		temp,0x02				;zaczenie przegldania stanu czujnikw (przerwanie Timera 0)
			out		TIMSK,temp
			sei
			ldi		delay,100				;pauza aby przerwanie zdyo odczyta czujniki
			rcall	delayms

alon_loop:
			ldi		xp,16					;odczytanie kodu dla zaczenia z klawiatury
			ldi		yp,1
			rcall	read_code
			ldi		eeaddr,OFFCODE_EEADDR
			rcall	compare_codes			;jeli kod odczytany jest identyczny z tym w eeprom,ustawiana jest fl.Z
			brne	alarm_on_ww				;skok do powtrnego zaczenia alarmu ale bez oczekiwania

			ldi		eeaddr,STATUS_EEADDR	;pierwsze zaczenie alarmu
			ldi		eedata,ALARMOFF			;ustawienie statusu alarmu na OFF
			rcall	ee_write

			ret


;-----------------------------------------------------------------------------
;przepisanie danych z ROM do RAM,adres ROM w Z, adres RAM w Y
;funkcja pomocnicza dla "send_SMS"; na kocu tekstu oczekiwane jest EOD
;-----------------------------------------------------------------------------
move_rom2ram:
			lpm								;zaaduj do R0 dane z ROM
			mov		temp,r0					;przenie do temp w celu porwnania
			cpi		temp,EOD				;czy to znak koca acucha?
			brne	move_rom2_loop
			ret								;tak,koniec pracy funkcji
move_rom2_loop:
			st		y,temp					;nie,zwiksz wskaniki
			inc		r28						;i zapamitaj odczytany znak w RAM
			adiw	r30,1
			rjmp	move_rom2ram


;-----------------------------------------------------------------------------
;zapisanie znakw CR i LF pod adres wskazywany przez Y : Y+1
;funkcja pomocnicza dla "send_SMS"
;-----------------------------------------------------------------------------
st_EOL:
			inc		r28
			ldi		temp,CR					;znak CR
			st		y,temp
			inc		r28
			ldi		temp,LF					;znak LF
			st		y,temp
			ret


;-----------------------------------------------------------------------------
;wysanie zawarto "buffer" przez UART
;na kocu acucha spodziewany jest CR+LF
;-----------------------------------------------------------------------------
puts_buffer:
			ldi		zl,LOW(buffer)			;Z = adres bufora "buffer" (1-bajtowy),Y zawiera aktualny adres "buffer"
			clr		zh
			rcall	puts					;wysanie numeru telefonu do modemu GSM
			rcall	gets					;odpowied modemu
			ldi		delay,1
			rcall	delaysec				;przerwa
			ret


;-----------------------------------------------------------------------------
;wysanie SMS z powiadomieniem
;-----------------------------------------------------------------------------
send_SMS:	
			ldi		zl,LOW(send_sms_1<<1)	;Z = adres nastawy-polecenia w ROM
			ldi		zh,HIGH(send_sms_1<<1)
			rcall	putf					;wysanie polecenia
			rcall	gets					;pobranie odpowiedzi modemu

			ldi		zl,LOW(send_sms_2<<1)	;Z = adres napisu w ROM
			ldi		zh,HIGH(send_sms_2<<1)
			ldi		yl,LOW(buffer)			;Y = adres bufora w RAM
			clr		yh						;tu adresy RAM mog by tylko 1-bajtowe
			rcall	move_rom2ram			;przepisanie danych z ROM do RAM,adresy ROM w Z,RAM w Y

			ldi		eeaddr,SMSNUMBER_EEADDR	;teraz pobranie numeru telefonu z eeprom do "buffer",na ktry wskazuje Y
			rcall	ee_read2buf				;uwaga: zmieniany jest rejestr Y

			ldi		temp,0x22				;cudzysw zamykajcy
			st		y,temp
			rcall	st_EOL					;zapisanie znakw CR+LF do bufora
			rcall	puts_buffer				;wysanie zawartoci "buffer" przez UART
											;teraz kompozycja SMS do wysania
			ldi		zl,LOW(send_sms_3<<1)	;odczyt czci "staej" z ROM i EEPROM
			ldi		zh,HIGH(send_sms_3<<1)
			ldi		yl,LOW(buffer)
			clr		yh
			rcall	move_rom2ram
			
			ldi		eeaddr,SMSTEXT_EEADDR	;teraz odczyt opisu obiektu (30 znakw wpr.przez uytkownika)
			rcall	ee_read2buf
			ldi		temp,','				;do temp kod przecinka
			st		y,temp					;zapisanie go w buforze
			clr		temp					;wyliczenia adresu etykiety i jej odczyt (jeli czujnik jest aktywny,to bit
send_SMS_sen:
			inc		temp					;przyjmuje warto "0")
			clc								;kasowanie flagi "carry"
			ror		sensors					;przeniesienie najmodszego bitu do C
			brcs	send_SMS_sen			;jeli C jest "1",to wejcie nie jest aktywne
			mov		eeaddr,temp
			rcall	sen_ee_addr				;wyznaczenie adresu w eeprom
			inc		yl						;odczyt etykiety wejcia do bufora w RAM
			rcall	ee_read2buf

			ldi		temp,0x1A				;zapamitanie znaku "Ctrl+Z"
			st		y,temp

			rcall	st_EOL					;zapisanie CR+LF
			rcall	puts_buffer				;wysanie zawartoci "buffer" przez UART

			inc		smssent					;ustawienie znacznika wysania SMS
			
			ret


;-----------------------------------------------------------------------------
;alarm aktywny - sygnalizacja dwikowa, powiadomienie za pomoc SMS,
;oczekiwanie na kod wyczenia
;-----------------------------------------------------------------------------
alarm_active:
			mov		temp,goon
			subi	temp,DELAYTIME			;sprawdzenie,czy upyn ju czas opnienia
			brsh	active_and_on
			rjmp	active_exit
active_and_on:
			cli								;wyczenie przerwa - ju nie s potrzebne
			sbi		portb,ALARMOUT			;zaczenie wyjcia alarmu
			tst		smssent					;jeli wysyano ju SMS,to nie wysyaj powtrnie
			brne	active_exit
			rcall	send_SMS				;wysanie sms z powiadomieniem
active_exit:
			ret


;-----------------------------------------------------------------------------
;P R O G R A M   G   W N Y
;-----------------------------------------------------------------------------
RESET:
;stos
			ldi		temp,LOW(RAMEND)
			out		SPL,temp				;ustawienie adresu wskanika stosu
;porty
			ldi		temp,0xFF				;ustawienie trybw pracy portw
			out		ddrb,temp				;port b pracuje w caoci jako wyjciowy
			mov		sensors,temp			;inicjalizacja zmiennej zawierajcej stan sensorw
			ldi		temp,0x3F
			out		portb,temp
			ldi		temp,0xC0
			out		ddrd,temp				;linie 0,2..5 portu d pracuj jako wejciowe-reszta wejciowe
											;inicjalizacja Timera 0 (przerwania)
;przerwania
			ldi		temp,0x05
			out		TCCR0,temp				;preskaler timera 1 na 1024
			out		TCCR1B,temp				;preskaler timera 2 na 1024
			clr		temp
			out		TCCR1A,temp
			out		TIMSK,temp
			out		TCNT0,temp				;zawarto inicjujca Timer 0
			ldi		temp,TIM1RLDH
			out		TCNT1H,temp
			ldi		temp,TIM1RLDL
			out		TCNT1L,temp				;zawarto inicjujca Timer 1
			cli								;przerwania zabronione

			rcall 	uart_init				;nastawy transmisji szeregowej
			rcall	lcd_init				;inicjalizacja LCD

STATUS_CHECK:
			ldi		eeaddr,STATUS_EEADDR	;odczyt statusu alarmu po zaczeniu
			rcall	ee_read
			cpi		eedata,ALARMON			;sprawdzenie,czy status alarmu jest "ON"
			breq	STATUS_OK
			cpi		eedata,ALARMOFF			;sprawdzenie,czy "OFF"
			breq	STATUS_OK
			rcall	alarm_set				;jeli aden z powyszych,to naley wykona nastawy
STATUS_OK:
			rcall	lcd_clr_scr
			ldi		zl,LOW(welcome1<<1)		;pobranie modszego bajtu adresu napisu do rejstru Z
			ldi		zh,HIGH(welcome1<<1)	; - / / - starszego bajtu do Z
			rcall	lcd_put_text			;wywietlenie
			ldi		i,8
			ldi		char,'.'
ST_LOOP:
			ldi		delay,1					;wywietlenie komunikatu, ok.8 sekund
			rcall	delaysec
			rcall	lcd_putc
			dec		i
			brne	ST_LOOP
;----------- nastawy modemu --------------------------------------------------------
			ldi		zl,LOW(init_modem_1<<1)
			ldi		zh,HIGH(init_modem_1<<1)
			rcall	putf					;"ATZ" -> modem
			rcall	gets					;odpowied modemu
			ldi		delay,1					;czas na wykonanie "reset"
			rcall	delaysec				;delaysec(1)

			rcall	send_PIN				;wysanie numeru PIN do telefonu

			ldi		zl,LOW(init_modem_3<<1)
			ldi		zh,HIGH(init_modem_3<<1)
			rcall	putf					;"ATE0" -> modem
			rcall	gets

			ldi		zl,LOW(init_modem_5<<1)
			ldi		zh,HIGH(init_modem_5<<1)
			rcall	putf					;"AT+CPMS="SM"" -> modem
			rcall	gets

;----------- odczyt statusu i wejcie w procedur czuwania lub wyczenia ----------
main:
			ldi		eeaddr,STATUS_EEADDR	;odczyt statusu alarmu po zaczeniu
			rcall	ee_read

			cpi		eedata,ALARMOFF			;wyczony?
			brne	main_next
			cbi		portb,ALARMOUT			;wyczenie alarmu
			rcall	alarm_off
			rjmp	main
main_next:									;zaczony?
			rcall	alarm_on
			rjmp	main


;-----------------------------------------------------------------------------
;tablice z tekstami
;-----------------------------------------------------------------------------
;teksty do wywietlenia
welcome1:		.db		"ALARM+SMS",EOD
;teksty statusu
whenoff:		.db		"ALARM WYLACZONY",EOD
whenon:			.db		"ALARM AKTYWNY",EOD
;teksty menu nastaw
set_1:			.db		"Kod zalaczenia?",EOD
set_2:			.db		"Kod wylaczenia?",EOD
set_3:			.db		"Kod PIN modemu?",EOD
set_4:			.db		"Telefon dla SMS?",EOD
set_5:			.db		"Tekst SMS (30 zn.)?",EOD
set_6:			.db		"Tekst wej. (8 zn.)?",EOD
set_ok:			.db		"ENTER=OK,C=popraw   ",EOD
set_wait:		.db		"Czekaj...",EOD
;nastawy modemu GSM
init_modem_1:	.db		"ATZ",CR,LF	;<- ten tekst wysyany bdzie przez UART,dlatego koczy si CR+LF
init_modem_2:	.db		"AT+CPIN=",EOD	;<- ten tekst bdzie przetwarzany przed wysyk,dlatego koczy si EOD
init_modem_3:	.db		"ATE0",CR,LF
init_modem_5:	.db		"AT+CPMS=",0x22,"SM",0x22,CR,LF
send_sms_1:		.db		"AT+CMGF=1",CR,LF
send_sms_2:		.db		"AT+CMGS=",0x22,"+",EOD
send_sms_3:		.db		"ALARM ZALACZONY W OBIEKCIE ",EOD

;tablica znakw do wprowadzania tekstu, naciniciu klawisza odpowiada znak z tablicy
;kolumna 1 zawiera znaki dla 1 nacinicia, 2 dla 2 itd.
chartable:		.db		'0',' ','/','-'		;kody dla klawisza "0"
				.db		'1','A','B','C'		;kody dla klawisza "1"
				.db		'2','D','E','F'		;kody dla klawisza "2"
				.db		'3','G','H','I'		;kody dla klawisza "3"
				.db		'4','J','K','L'		;kody dla klawisza "4"
				.db		'5','M','N','O'		;kody dla klawisza "5"
				.db		'6','P','Q','R'		;kody dla klawisza "6"
				.db		'7','S','T','U'		;kody dla klawisza "7"
				.db		'8','V','W','Z'		;kody dla klawisza "8"
				.db		'9','Y','Z','.'		;kody dla klawisza "9"
